#include <avr/io.h>
#include <util/delay.h>

#define output(pin) (PORTA.DIR |= pin) // set port direction for output
#define input(pin) (PORTA.DIR &= (~pin)) // set port direction for input
#define set(pin) (PORTA.OUT |= pin) // set port pin
#define clear(pin) (PORTA.OUT &= (~pin)) // clear port pin
#define pin_test(pin) (PORTA.IN & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define bit_delay_time 102 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay

#define serial_pin_in (1 << 6) //TXD
#define serial_pin_out (1 << 7) //RXD
#define max_buffer 25

void get_char(unsigned char pin, char *rxbyte) {
  //
  // read character into rxbyte on pins pin
  //    assumes line driver (inverts bits)
  //
  *rxbyte = 0;
  while (pin_test(pin))
    //
    // wait for start bit
    //
    ;
  //
  // delay to middle of first data bit
  //
  half_bit_delay();
  bit_delay();
  //
  // unrolled loop to read data bits
  //
  if pin_test(pin)
    *rxbyte |= (1 << 0);
  else
    *rxbyte |= (0 << 0);
  bit_delay();
  if pin_test(pin)
    *rxbyte |= (1 << 1);
  else
    *rxbyte |= (0 << 1);
  bit_delay();
  if pin_test(pin)
    *rxbyte |= (1 << 2);
  else
    *rxbyte |= (0 << 2);
  bit_delay();
  if pin_test(pin)
    *rxbyte |= (1 << 3);
  else
    *rxbyte |= (0 << 3);
  bit_delay();
  if pin_test(pin)
    *rxbyte |= (1 << 4);
  else
    *rxbyte |= (0 << 4);
  bit_delay();
  if pin_test(pin)
    *rxbyte |= (1 << 5);
  else
    *rxbyte |= (0 << 5);
  bit_delay();
  if pin_test(pin)
    *rxbyte |= (1 << 6);
  else
    *rxbyte |= (0 << 6);
  bit_delay();
  if pin_test(pin)
    *rxbyte |= (1 << 7);
  else
    *rxbyte |= (0 << 7);
  //
  // wait for stop bit
  //
  bit_delay();
  half_bit_delay();
}

void put_char(unsigned char pin, char txchar) {
  //
  // send character in txchar on port pin
  //    assumes line driver (inverts bits)
  //
  // start bit
  //
  clear(pin);
  bit_delay();
  //
  // unrolled loop to write data bits
  //
  if bit_test(txchar,0)
    set(pin);
  else
    clear(pin);
  bit_delay();
  if bit_test(txchar,1)
    set(pin);
  else
    clear(pin);
  bit_delay();
  if bit_test(txchar,2)
    set(pin);
  else
    clear(pin);
  bit_delay();
  if bit_test(txchar,3)
    set(pin);
  else
    clear(pin);
  bit_delay();
  if bit_test(txchar,4)
    set(pin);
  else
    clear(pin);
  bit_delay();
  if bit_test(txchar,5)
    set(pin);
  else
    clear(pin);
  bit_delay();
  if bit_test(txchar,6)
    set(pin);
  else
    clear(pin);
  bit_delay();
  if bit_test(txchar,7)
    set(pin);
  else
    clear(pin);
  bit_delay();
  //
  // stop bit
  //
  set(pin);
  bit_delay();
  //
  // char delay
  //
  bit_delay();
}

void put_string(unsigned char pin, String str) {
  //
  // print a null-terminated string
  //
  static int index;
  index = 0;
  do {
    put_char(pin, str[index]);
    ++index;
    } while (str[index] != 0);
}

//static char chr;
static char buffer[max_buffer] = {0};
static int index = 0;

void setup() {
  set(serial_pin_out);
  output(serial_pin_out);
}
void loop() {
  char curr;
  while (curr != "\n") {
    get_char(serial_pin_in, &curr);
    buffer[index] = curr;
    index++;
  }
  put_string(serial_pin_out, buffer);
}
